import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { Observable, of, merge, concat } from 'rxjs';
import { Employee } from 'src/typings';
import { map, flatMap, tap, scan } from 'rxjs/operators';


const MAX_PAGE_SIZE = 50;

type Chunk = {
  data: Employee[]
  idx: number
};

@Injectable({
  providedIn: 'root'
})
export class EmployeesService {

  constructor(private http: HttpClient) { }

  getEmployees(page = 1) {
    return this.http.get<Employee[]>(`http://localhost:3000/employees?_page=${page}`);
  }

  getAllEmployees(): Observable<Employee[]> {
    return this.getFirstPage().pipe(
      flatMap(({ total, data }) => {
        const pageCount = Math.ceil(total / MAX_PAGE_SIZE);
        const pages = Array(pageCount - 1).fill(0).map((_, idx) => idx + 2);
        return merge(
          of({ data, idx: 1 }),
          ...pages.map(idx => this.getEmployees(idx).pipe(
            map(data => ({ data, idx }))
          ))
        )
      }),
      scan((sparseChunks, { data, idx }: Chunk) => {
        sparseChunks[idx] = data
        return sparseChunks;
      }, [] as Employee[][]),
      map(sparseChunks => sparseChunks.flat())
      // scan((allItems, chunk: Chunk) => allItems.concat(chunk.data), [] as Employee[] ),

    );
  }

  private getFirstPage(): Observable<{ total: number, data: Employee[] }> {
    return this.http.get<Employee[]>(`http://localhost:3000/employees`, { observe: 'response' }).pipe(
      map(res => ({
        total: parseInt(res.headers.get('X-Total-Count')),
        data: res.body,
      })),
    );
  }

}
